function [co] = HTsallis_HRenyi_initialization(mult)
%Initialization of the meta Tsallis entropy estimator defined according to the relation: H_{T,alpha} = (e^{H_{R,alpha}(1-alpha)} - 1) / (1-alpha), where H_{T,alpha} and H_{R,alpha} denotes the Tsallis and the Renyi entropy, respectively.
%
%Note:
%   1)The estimator is treated as a cost object (co).
%   2)We make use of the naming convention 'H<name>_initialization', to ease embedding new entropy estimation methods.
%
%INPUT:
%   mult: is a multiplicative constant relevant (needed) in the estimation; '=1' means yes, '=0' no.
%OUTPUT:
%   co: cost object (structure).
%
%Copyright (C) 2012 Zoltan Szabo ("http://nipg.inf.elte.hu/szzoli", "szzoli (at) cs (dot) elte (dot) hu")
%
%This file is part of the ITE (Information Theoretical Estimators) Matlab/Octave toolbox.
%
%ITE is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
%
%This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with ITE. If not, see <http://www.gnu.org/licenses/>.

%mandatory fields:
    co.name = 'Tsallis_HRenyi';
    co.mult = mult;
    
%other fields:
    co.alpha = 0.95; 

    co.member_name = 'Renyi_kNN_k'; %you can change it to any Renyi entropy estimator
    co.member_co = H_initialization(co.member_name,mult);
    co.member_co.alpha = co.alpha;
